# IOPSYS RPCD Patches

## Login Guards
Login guards are a tool for brute force prevention. It will lock out a user for
a specified time, after a certain amount of failed login attempts.

The feature depends on a section ```brute-force```, provided in
```/etc/config/rpcd```. In the following example section 5 incorrect login
attempts are allowed before locking a user out, if no login attempts are seen
for 10 seconds, the attempt counter is reset. If 5 consecutive incorrect
attempts are reached the user will be locked out for 15 seconds.
```
config brute-force
	option max_incorrect_attempts '5'	# (seconds) maximum number of incorrect login attempts before lockout
	option max_time_before_locking '10'	# (seconds) maximum time period where number of incorrect login attempts are required before locking the account
	option account_lock_duration '15'	# (seconds) lockout duration after reaching maximum number of incorrect attempts.
```

These options do not have any default value set, meaning if any option is
missing, or given with incorrect types (i.e. characters), the login guard will
not be used.

### Login Guard example

```
root@iopsys:~# cat /etc/config/rpcd
config brute-force
	option max_incorrect_attempts '2'
	option max_time_before_locking '2'
	option account_lock_duration '2'

...

root@iopsys:~# cat test.sh
ubus call session login '{"username":"user", "password":"incorrect"}'
ubus call session login '{"username":"user", "password":"incorrect"}'

ubus call session login '{"username":"user", "password":"user"}' #should fail
echo sleeping
sleep 2
ubus call session login '{"username":"user", "password":"user"}' #should succeed

root@iopsys:~#
root@iopsys:~#
root@iopsys:~#
root@iopsys:~# sh test.sh
Command failed: Permission denied
Command failed: Permission denied
Command failed: Permission denied
sleeping
{
	"ubus_rpc_session": "76e61637adeef121fe4ecc4bebcf6458",
	...
}
```

## Granular UCI Access Control
The granular uci access control offers increased control over what users are
able to see, add or change through RPCD's ```uci``` object, published on ubus.

The UCI granular control is implemented ontop of the existing RPCD access
control list functionality (```/usr/share/rpcd/acl.d/```). Alongside the keys
```uci``` and ```ubus```, at the ```read``` or ```write``` level in the json,
support for ```uci_granular``` has been added.

```
{
	"user": {
		"description": "User access",
		"read": {
			"uci": [
				"samba"
				"wireless",
			]
		},
		"write": {
			"ubus": {
				...
			},
			"uci": [
				"wireless",
			],
			"uci_granular": {
				...
			}
		}
	}
}

```

In the ```uci_granular``` object, configuration file names are used as keys to
arrays, holding permission objects for the corresponding configuration file.

```
"uci_granular": {
		"wireless": [
			...
		],
		"network": [
			...
		]
	}
```

A permission object should contain a ```match``` object, specifying what
options, section types or section names (or any combination thereof) it should
match on. Secondly, a permission object can contain an ```option``` array,
specifying which options that should be made accessible. If an option array is
not specified (or contains ```*```), unrestricted access is granted to
matching sections, meaning the user may get, set, add, delete etc. freely
(assuming the access control list is provided as a ```write``` list). If options
are listed, the user will only be able to get or set those options, and have no
other section access.

```
"wireless": [
	{
		"match": {
			".type": "wifi-iface",
			"network": "lan"
		},
		"option": [
			"ssid",
			"key"
		],
		"match" : {
			".type" : "wifi-device"
			},
			"option" : [
				"*"
			]
		}
	},
```

It is possible for a user to belong to several access groups with unique access
restrictions, in which case it will use the union of the restrictions.

### RPCD example
```
root@iopsys:~# cat /usr/share/rpcd/user.json
{
        "user": {
                "description": "End user access role",
                "write": {
                        "ubus": {
                                "router.wps": [
                                        "pbc",
                                        "status",
                                        "stop"
                                ],
                                "router.wireless": [
                                        "scan",
                                        "scanresults",
                                        "status",
                                        "stas"
                                ]
                        },
                        "uci": [
                                "samba",
                                "wireless",
                                "network"
                        ],
                        "owsd": [
                                "wps",
                                "wifi.wps",
                                "wifi-repeater-success"
                        ],
                        "uci_granular":{
                          "wireless" : [
                            {
                              "match" : {
                                ".type" : "wifi-iface",
                                "network" : "lan"
                              },
                              "option" : [
                                "ssid",
                                "key"
                              ]
                            },
                            {
                              "match" : {
                                ".type" : "wifi-device"
                              },
                              "option" : [
                                "channel"
                              ]
                            }
                          ],
                          "network" : [
                            {
                              "match" : {
                                ".name" : "lan",
                              },
                              "option" : [
                                "*"
                              ]
                            },
                            {
                              "match" : {
                                ".type" : "device"
                              }
                            }
                          ]
                        }
                }
        }
}
root@iopsys:~# cat /etc/config/rpcd
config login
		option username 'user'
		option password '$p$user'
		list write 'user'

root@iopsys:~# ubus call session login '{"username":"user", "password":"user"}'
| grep rpc
    "ubus_rpc_session": "5270d2bdb1f89805bcb0a8cc7da9f964",

root@iopsys:~# ubus call uci get '{"config":"wireless", "ubus_rpc_session":
"5270d2bdb1f89805bcb0a8cc7da9f964"}'
{
    "values": {
        "wl0": {
            ".anonymous": false,
            ".type": "wifi-device",
            ".name": "wl0",
            ".index": 3,
            "channel": "36"
        },
        "cfg063579": {
            ".anonymous": true,
            ".type": "wifi-iface",
            ".name": "cfg063579",
            ".index": 4,
            "ssid": "iopsys-4C3A",
            "key": "55RKNXNCCCCGSC"
        },
        "wl1": {
            ".anonymous": false,
            ".type": "wifi-device",
            ".name": "wl1",
            ".index": 5,
            "channel": "36"
        },
        "cfg093579": {
            ".anonymous": true,
            ".type": "wifi-iface",
            ".name": "cfg093579",
            ".index": 6,
            "key": "55RKNXNCCCCGSC",
            "ssid": "test"
        }
    }
}
```
## Login Control
An option, ```option enabled```, has been added to ```/etc/config/rpcd``` under
```config login``` sections, making it possible to prevent a user from
logging in via rpcd (defaulting to ```enabled '1'``` if option is unspecified).


### Login Control Example
```
root@iopsys:~# cat /etc/config/rpcd
config login
	option username 'user'
	option password '$p$user'
	option enabled '0'
	list write 'user'
root@iopsys:~# ubus call session login '{"username":"user", "password":"user"}'
Command failed: Permission denied
root@iopsys:~# uci set rpcd.@login[-1].enabled=1
root@iopsys:~# uci commit
root@iopsys:~# /etc/init.d/rpcd restart
root@iopsys:~# ubus call session login '{"username":"user", "password":"user"}'
{
	"ubus_rpc_session": "a9abd1bb1aa78f71f1387438b36d8acd",
	...
}
root@iopsys:~# uci delete rpcd.@login[-1].enabled
root@iopsys:~# uci commit
root@iopsys:~# /etc/init.d/rpcd restart
root@iopsys:~# ubus call session login '{"username":"user", "password":"user"}'
{
	"ubus_rpc_session": "8df19cc2e6c9dc682e91344e95942295",
	...
}
```
